
# Modified from: http://www2.yukawa.kyoto-u.ac.jp/~kyohei.kawaguchi/kn_calc_bns1/main.html
# Reference: Dietrich et al. http://adsabs.harvard.edu/cgi-bin/bib_query?arXiv:1612.03665

import numpy as np
import scipy

from .model import register_model
from .. import KNTable

from gwemlightcurves.EjectaFits.DiUj2017 import calc_meje, calc_vej

def get_DiUj2017_model(table, **kwargs):
    if not 'mej' in table.colnames:
        # calc the mass of ejecta
        table['mej'] = calc_meje(table['m1'], table['mb1'], table['c1'], table['m2'], table['mb2'], table['c2'])
        # calc the velocity of ejecta
        table['vej'] = calc_vej(table['m1'],table['c1'],table['m2'],table['c2'])

    # Throw out smaples where the mass ejecta is less than zero.
    mask = (table['mej'] > 0)
    table = table[mask]
    if len(table) == 0: return table

    # Log mass ejecta
    table['mej10'] = np.log10(table['mej'])
    # Initialize columns
    timeseries = np.arange(table['tini'][0], table['tmax'][0]+table['dt'][0], table['dt'][0])
    table['t'] = [np.zeros(timeseries.size)]
    table['lbol'] = [np.zeros(timeseries.size)]
    table['mag'] =  [{}]

    # Loop over samples
    for isample in range(len(table)):
        table['t'][isample], table['lbol'][isample], table['mag'][isample] = calc_lc(table['tini'][isample],table['tmax'][isample],
                                                      table['dt'][isample],table['mej'][isample],table['vej'][isample],table['vmin'][isample],
                                                      table['th'][isample],table['ph'][isample], table['kappa'][isample],
                                                      table['eps'][isample],table['alp'][isample],table['eth'][isample],table['flgbct'][isample])
    return table

def calc_lc(tini,tmax,dt,mej,vej,vmin,th,ph,kappa,eps,alp,eth,flgbct):

    td, bct = setbc_tabular()
    bc = setbc()

    lbol_d=[]
    bc_tmp=[]
    dc=0

    t_d = np.arange(tini,tmax+dt,dt)

    mag_d = {}
    for ii in range(8):
        mag_d[ii] = np.array([])

    epsBarnes = 0
    if epsBarnes:
        mejtab = (0.001,0.001,0.001,0.005,0.005,0.005,0.01,0.01,0.01,0.05,0.05,0.05)
        vejtab = (0.1,0.2,0.3,0.1,0.2,0.3,0.1,0.2,0.3,0.1,0.2,0.3)
        a      = (2.01,4.52,8.16,0.81,1.90,3.20,0.56,1.31,2.19,0.27,0.55,0.95)
        b      = (0.28,0.62,1.19,0.19,0.28,0.45,0.17,0.21,0.31,0.10,0.13,0.15)
        d      = (1.12,1.39,1.52,0.86,1.21,1.39,0.74,1.13,1.32,0.60,0.90,1.13)

        fa     = scipy.interpolate.interp2d(mejtab,vejtab,a, kind='linear')
        fb     = scipy.interpolate.interp2d(mejtab,vejtab,b, kind='linear')
        fd     = scipy.interpolate.interp2d(mejtab,vejtab,d, kind='linear')

        fam    = fa(mej,vej)
        fbm    = fb(mej,vej)
        fdm    = fd(mej,vej)

    for t in t_d:
      if epsBarnes:
          eth = 0.36*(np.exp(-fam*t)+ np.log(1+2*fbm*t**fdm)/(2*fbm*t**fdm))[0]

      lbol=kn_lbol(t,mej,vej,vmin,th,ph,kappa,eps,alp,eth)
      lbol_d.append(lbol)
      mbol=mag_bol(lbol,10)
      tt=t/((mej*100.0)**(1.0/3.2))
      bc_tmp=getBC(td,bc,bct,tt,flgbct)

      for ii in range(8):
        if t > 2.*(mej*100)**(1.0/3.2):
          mag_d[ii] = np.append(mag_d[ii],mbol-bc_tmp[ii])
        else:
          mag_d[ii] = np.append(mag_d[ii],np.nan)

    lbol_d = np.array(lbol_d)

    wavelengths = [3543, 4775.6, 6129.5, 7484.6, 8657.8, 12350, 16620, 21590]
    wavelength_interp = 9603.1

    mag_y = np.zeros(t_d.shape)
    for ii in range(len(t_d)):
        mags = [mag_d[jj][ii] for jj in range(8)]
        mag_y[ii] = np.interp(wavelength_interp,wavelengths,mags)
    mag_new = {}
    mag_new[0] = mag_d[0]
    mag_new[1] = mag_d[1]
    mag_new[2] = mag_d[2]
    mag_new[3] = mag_d[3]
    mag_new[4] = mag_d[4]
    mag_new[5] = mag_y
    mag_new[6] = mag_d[5]
    mag_new[7] = mag_d[6]
    mag_new[8] = mag_d[7]

    return t_d, lbol_d, mag_new

def mag_bol(lbol,d):
  f0=2.52e-5
  pc=3.08568e18
  d0=d*pc

  return -2.5*np.log(lbol/4/np.pi/d0/d0/f0)/np.log(10.0)

def getBC(td,bc,bct,tt,flgbct):

  if flgbct:
    if (tt<td[0]) or (tt>td[130]):
        return np.array([np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan])
    else:
      for ii in range(130):
          if (td[ii]<=tt) and (tt<=td[ii+1]):
              break
      bc_tmp=np.zeros((8,))
      fac=(tt-td[ii])/(td[ii+1]-td[ii])
      for jj in range(8):
          bc_tmp[jj]=(1-fac)*bct[jj][ii]+fac*bct[jj][ii+1]
          if not np.isfinite(bc_tmp[jj]):
              bc_tmp[jj] = np.nan
  else:
    if (tt<2) or (tt>15):
        return np.array([np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan])
    else:
      bc_tmp=np.zeros((8,))
      for jj in range(8):
          if (jj == 0) and (tt>5):
            bc_tmp[jj]=np.nan
          elif (jj == 1) and (tt>8.5):
            bc_tmp[jj]=np.nan
          else:
            bc_tmp[jj]=bc[jj][0]+bc[jj][1]*tt+bc[jj][2]*(tt**2.0)+bc[jj][3]*(tt**3.0)+bc[jj][4]*(tt**4.0)
          if not np.isfinite(bc_tmp[jj]):
            bc_tmp[jj] = np.nan

  return bc_tmp

def kn_lbol(t,mej,vej,vmin,th,ph,kappa,eps,alp,eth):
  c=2.99792458e10
  msun=1.988e33
  day=24*3600
  lu0=c*day
  eneu0=msun*c*c
  lumu0=eneu0/day
  kappa0=kappa/lu0/lu0*msun
  eps0=eth*eps/eneu0*day*msun

  vdiff = vmax(vej,vmin)-vmin
  if vdiff < 0:
      tobs = 0.0
  else:
      tobs=(th*mej*kappa0/(2*ph*vdiff))**(1/2.0)

  if (t<tobs):
      fac=t/tobs
  else:
      fac=1

  lbol=(1+th)*mej*fac*eps0*(t**(-alp))*lumu0

  return lbol

def vmax(vej,vmin):
    return 2.0*vej-vmin

def setbc():
    bc = np.zeros((8,5))

    # bc u-band
    bc[0][0]= 40.01
    bc[0][1]=-56.79
    bc[0][2]= 25.73
    bc[0][3]=-5.207
    bc[0][4]=0.3813

    # bc g-band
    bc[1][0]=-6.195
    bc[1][1]=4.054
    bc[1][2]=-1.754
    bc[1][3]=0.2246
    bc[1][4]=-0.009813

    # bc r-band
    bc[2][0]=-2.308
    bc[2][1]=1.445
    bc[2][2]=-0.5740
    bc[2][3]=0.0531
    bc[2][4]=-0.00152

    # bc i-band
    bc[3][0]=0.6441
    bc[3][1]=0.0796
    bc[3][2]=-0.122
    bc[3][3]=0.00793
    bc[3][4]=-0.000122

    # bc z-band
    bc[4][0]=1.072
    bc[4][1]=0.3646
    bc[4][2]=-0.1032
    bc[4][3]=0.00368
    bc[4][4]=0.0000126

    # bc J-band
    bc[5][0]=-1.038
    bc[5][1]=1.348
    bc[5][2]=-0.2364
    bc[5][3]=0.0137
    bc[5][4]=-0.000261

    # bc H-band
    bc[6][0]=-2.763
    bc[6][1]=1.502
    bc[6][2]=-0.2133
    bc[6][3]=0.0128
    bc[6][4]=-0.000288

    # bc K-band
    bc[7][0]=-7.876
    bc[7][1]=3.245
    bc[7][2]=-0.3946
    bc[7][3]=0.0216
    bc[7][4]=-0.000443

    return bc

def setbc_tabular():
    td= np.zeros((131,))
    bct = np.zeros((8,131))

    # Rescaled time
    td[0]=2.
    td[1]=2.1
    td[2]=2.2
    td[3]=2.3
    td[4]=2.4
    td[5]=2.5
    td[6]=2.6
    td[7]=2.7
    td[8]=2.8
    td[9]=2.9
    td[10]=3.
    td[11]=3.1
    td[12]=3.2
    td[13]=3.3
    td[14]=3.4000000000000004
    td[15]=3.5
    td[16]=3.6
    td[17]=3.7
    td[18]=3.8
    td[19]=3.9000000000000004
    td[20]=4.
    td[21]=4.1
    td[22]=4.2
    td[23]=4.300000000000001
    td[24]=4.4
    td[25]=4.5
    td[26]=4.6
    td[27]=4.7
    td[28]=4.800000000000001
    td[29]=4.9
    td[30]=5.
    td[31]=5.1
    td[32]=5.2
    td[33]=5.300000000000001
    td[34]=5.4
    td[35]=5.5
    td[36]=5.6
    td[37]=5.7
    td[38]=5.800000000000001
    td[39]=5.9
    td[40]=6.
    td[41]=6.1000000000000005
    td[42]=6.2
    td[43]=6.3
    td[44]=6.4
    td[45]=6.5
    td[46]=6.6000000000000005
    td[47]=6.7
    td[48]=6.800000000000001
    td[49]=6.9
    td[50]=7.
    td[51]=7.1000000000000005
    td[52]=7.2
    td[53]=7.300000000000001
    td[54]=7.4
    td[55]=7.5
    td[56]=7.6000000000000005
    td[57]=7.7
    td[58]=7.800000000000001
    td[59]=7.9
    td[60]=8.
    td[61]=8.100000000000001
    td[62]=8.2
    td[63]=8.3
    td[64]=8.4
    td[65]=8.5
    td[66]=8.600000000000001
    td[67]=8.7
    td[68]=8.8
    td[69]=8.9
    td[70]=9.
    td[71]=9.100000000000001
    td[72]=9.2
    td[73]=9.3
    td[74]=9.4
    td[75]=9.5
    td[76]=9.600000000000001
    td[77]=9.7
    td[78]=9.8
    td[79]=9.9
    td[80]=10.
    td[81]=10.1
    td[82]=10.200000000000001
    td[83]=10.3
    td[84]=10.4
    td[85]=10.5
    td[86]=10.6
    td[87]=10.700000000000001
    td[88]=10.8
    td[89]=10.9
    td[90]=11.
    td[91]=11.1
    td[92]=11.200000000000001
    td[93]=11.3
    td[94]=11.4
    td[95]=11.5
    td[96]=11.600000000000001
    td[97]=11.700000000000001
    td[98]=11.8
    td[99]=11.9
    td[100]=12.
    td[101]=12.100000000000001
    td[102]=12.200000000000001
    td[103]=12.3
    td[104]=12.4
    td[105]=12.5
    td[106]=12.600000000000001
    td[107]=12.700000000000001
    td[108]=12.8
    td[109]=12.9
    td[110]=13.
    td[111]=13.100000000000001
    td[112]=13.200000000000001
    td[113]=13.3
    td[114]=13.4
    td[115]=13.5
    td[116]=13.600000000000001
    td[117]=13.700000000000001
    td[118]=13.8
    td[119]=13.9
    td[120]=14.
    td[121]=14.100000000000001
    td[122]=14.200000000000001
    td[123]=14.3
    td[124]=14.4
    td[125]=14.5
    td[126]=14.600000000000001
    td[127]=14.700000000000001
    td[128]=14.8
    td[129]=14.9
    td[130]=15.

    bct[0][0]=-6.395022155360145
    bct[0][1]=-6.654952354994068
    bct[0][2]=-6.888948589861781
    bct[0][3]=-7.082062657258129
    bct[0][4]=-7.275767561238976
    bct[0][5]=-7.48698283376036
    bct[0][6]=-7.704254284905159
    bct[0][7]=-7.909255274422776
    bct[0][8]=-8.136286787029755
    bct[0][9]=-8.397831990382068
    bct[0][10]=-8.676429532903112
    bct[0][11]=-8.95916934213983
    bct[0][12]=-9.190273332845704
    bct[0][13]=-9.3847091829365
    bct[0][14]=-9.577094163669008
    bct[0][15]=-9.757180803139065
    bct[0][16]=-9.930948502387928
    bct[0][17]=-10.079303190628535
    bct[0][18]=-10.22765787886914
    bct[0][19]=-10.45053344660461
    bct[0][20]=-10.736186403227379
    bct[0][21]=-11.229283785395099
    bct[0][22]=-11.791159795420144
    bct[0][23]=-12.322140558677862
    bct[0][24]=-12.773059471044062
    bct[0][25]=-13.203697347917487
    bct[0][26]=-13.24431876213202
    bct[0][27]=-13.186714579098433
    bct[0][28]=-13.130111517871207
    bct[0][29]=-13.231804633973873
    bct[0][30]=-13.338954121107179
    bct[0][31]=-np.inf
    bct[0][32]=-np.inf
    bct[0][33]=-np.inf
    bct[0][34]=-np.inf
    bct[0][35]=-np.inf
    bct[0][36]=-np.inf
    bct[0][37]=-np.inf
    bct[0][38]=-np.inf
    bct[0][39]=-np.inf
    bct[0][40]=-np.inf
    bct[0][41]=-np.inf
    bct[0][42]=-np.inf
    bct[0][43]=-np.inf
    bct[0][44]=-np.inf
    bct[0][45]=-np.inf
    bct[0][46]=-np.inf
    bct[0][47]=-np.inf
    bct[0][48]=-np.inf
    bct[0][49]=-np.inf
    bct[0][50]=-np.inf
    bct[0][51]=-np.inf
    bct[0][52]=-np.inf
    bct[0][53]=-np.inf
    bct[0][54]=-np.inf
    bct[0][55]=-np.inf
    bct[0][56]=-np.inf
    bct[0][57]=-np.inf
    bct[0][58]=-np.inf
    bct[0][59]=-np.inf
    bct[0][60]=-np.inf
    bct[0][61]=-np.inf
    bct[0][62]=-np.inf
    bct[0][63]=-np.inf
    bct[0][64]=-np.inf
    bct[0][65]=-np.inf
    bct[0][66]=-np.inf
    bct[0][67]=-np.inf
    bct[0][68]=-np.inf
    bct[0][69]=-np.inf
    bct[0][70]=-np.inf
    bct[0][71]=-np.inf
    bct[0][72]=-np.inf
    bct[0][73]=-np.inf
    bct[0][74]=-np.inf
    bct[0][75]=-np.inf
    bct[0][76]=-np.inf
    bct[0][77]=-np.inf
    bct[0][78]=-np.inf
    bct[0][79]=-np.inf
    bct[0][80]=-np.inf
    bct[0][81]=-np.inf
    bct[0][82]=-np.inf
    bct[0][83]=-np.inf
    bct[0][84]=-np.inf
    bct[0][85]=-np.inf
    bct[0][86]=-np.inf
    bct[0][87]=-np.inf
    bct[0][88]=-np.inf
    bct[0][89]=-np.inf
    bct[0][90]=-np.inf
    bct[0][91]=-np.inf
    bct[0][92]=-np.inf
    bct[0][93]=-np.inf
    bct[0][94]=-np.inf
    bct[0][95]=-np.inf
    bct[0][96]=-np.inf
    bct[0][97]=-np.inf
    bct[0][98]=-np.inf
    bct[0][99]=-np.inf
    bct[0][100]=-np.inf
    bct[0][101]=-np.inf
    bct[0][102]=-np.inf
    bct[0][103]=-np.inf
    bct[0][104]=-np.inf
    bct[0][105]=-np.inf
    bct[0][106]=-np.inf
    bct[0][107]=-np.inf
    bct[0][108]=-np.inf
    bct[0][109]=-np.inf
    bct[0][110]=-np.inf
    bct[0][111]=-np.inf
    bct[0][112]=-np.inf
    bct[0][113]=-np.inf
    bct[0][114]=-np.inf
    bct[0][115]=-np.inf
    bct[0][116]=-np.inf
    bct[0][117]=-np.inf
    bct[0][118]=-np.inf
    bct[0][119]=-np.inf
    bct[0][120]=-np.inf
    bct[0][121]=-np.inf
    bct[0][122]=-np.inf
    bct[0][123]=-np.inf
    bct[0][124]=-np.inf
    bct[0][125]=-np.inf
    bct[0][126]=-np.inf
    bct[0][127]=-np.inf
    bct[0][128]=-np.inf
    bct[0][129]=-np.inf
    bct[0][130]=-np.inf

    bct[1][0]=-3.3222333743148966
    bct[1][1]=-3.4381699552906664
    bct[1][2]=-3.5555353140699872
    bct[1][3]=-3.682657941103013
    bct[1][4]=-3.8152758253989987
    bct[1][5]=-3.956858845211015
    bct[1][6]=-4.094531520516839
    bct[1][7]=-4.228014215191751
    bct[1][8]=-4.362142790430276
    bct[1][9]=-4.495263558325909
    bct[1][10]=-4.631451065673255
    bct[1][11]=-4.766010285167987
    bct[1][12]=-4.9128008578303515
    bct[1][13]=-5.058572615733952
    bct[1][14]=-5.203908239405083
    bct[1][15]=-5.351749661917828
    bct[1][16]=-5.498440827816225
    bct[1][17]=-5.64339808282588
    bct[1][18]=-5.788355337835537
    bct[1][19]=-5.938829914010338
    bct[1][20]=-6.095729373895152
    bct[1][21]=-6.268681806949762
    bct[1][22]=-6.445849530968997
    bct[1][23]=-6.621899009541406
    bct[1][24]=-6.788702616511063
    bct[1][25]=-6.948231713330248
    bct[1][26]=-7.110141746737961
    bct[1][27]=-7.273603831950621
    bct[1][28]=-7.437062770804548
    bct[1][29]=-7.602513871008245
    bct[1][30]=-7.776862263612658
    bct[1][31]=-7.944425594529796
    bct[1][32]=-8.097683224675071
    bct[1][33]=-8.250940854820346
    bct[1][34]=-8.409116295486266
    bct[1][35]=-8.588716965811205
    bct[1][36]=-8.770296637747803
    bct[1][37]=-8.93932770304942
    bct[1][38]=-9.097453506841612
    bct[1][39]=-9.2555793106338
    bct[1][40]=-9.413705114425989
    bct[1][41]=-9.548278144020891
    bct[1][42]=-9.638654247208159
    bct[1][43]=-9.727848158603251
    bct[1][44]=-9.81460177173779
    bct[1][45]=-9.878274374309044
    bct[1][46]=-9.941946976880303
    bct[1][47]=-10.005619579451556
    bct[1][48]=-10.060813216597698
    bct[1][49]=-10.094082353315736
    bct[1][50]=-10.130683010869419
    bct[1][51]=-10.188294195145389
    bct[1][52]=-10.301751271403006
    bct[1][53]=-10.425026121958318
    bct[1][54]=-10.548300972513625
    bct[1][55]=-10.671575823068935
    bct[1][56]=-10.794850673624243
    bct[1][57]=-10.961100132971934
    bct[1][58]=-11.092637145584032
    bct[1][59]=-11.18737188535205
    bct[1][60]=-11.285854552348448
    bct[1][61]=-11.3844133415299
    bct[1][62]=-11.481520090762153
    bct[1][63]=-11.578626839994412
    bct[1][64]=-11.675733589226668
    bct[1][65]=-11.772840338458924
    bct[1][66]=-np.inf
    bct[1][67]=-np.inf
    bct[1][68]=-np.inf
    bct[1][69]=-np.inf
    bct[1][70]=-np.inf
    bct[1][71]=-np.inf
    bct[1][72]=-np.inf
    bct[1][73]=-np.inf
    bct[1][74]=-np.inf
    bct[1][75]=-np.inf
    bct[1][76]=-np.inf
    bct[1][77]=-np.inf
    bct[1][78]=-np.inf
    bct[1][79]=-np.inf
    bct[1][80]=-np.inf
    bct[1][81]=-np.inf
    bct[1][82]=-np.inf
    bct[1][83]=-np.inf
    bct[1][84]=-np.inf
    bct[1][85]=-np.inf
    bct[1][86]=-np.inf
    bct[1][87]=-np.inf
    bct[1][88]=-np.inf
    bct[1][89]=-np.inf
    bct[1][90]=-np.inf
    bct[1][91]=-np.inf
    bct[1][92]=-np.inf
    bct[1][93]=-np.inf
    bct[1][94]=-np.inf
    bct[1][95]=-np.inf
    bct[1][96]=-np.inf
    bct[1][97]=-np.inf
    bct[1][98]=-np.inf
    bct[1][99]=-np.inf
    bct[1][100]=-np.inf
    bct[1][101]=-np.inf
    bct[1][102]=-np.inf
    bct[1][103]=-np.inf
    bct[1][104]=-np.inf
    bct[1][105]=-np.inf
    bct[1][106]=-np.inf
    bct[1][107]=-np.inf
    bct[1][108]=-np.inf
    bct[1][109]=-np.inf
    bct[1][110]=-np.inf
    bct[1][111]=-np.inf
    bct[1][112]=-np.inf
    bct[1][113]=-np.inf
    bct[1][114]=-np.inf
    bct[1][115]=-np.inf
    bct[1][116]=-np.inf
    bct[1][117]=-np.inf
    bct[1][118]=-np.inf
    bct[1][119]=-np.inf
    bct[1][120]=-np.inf
    bct[1][121]=-np.inf
    bct[1][122]=-np.inf
    bct[1][123]=-np.inf
    bct[1][124]=-np.inf
    bct[1][125]=-np.inf
    bct[1][126]=-np.inf
    bct[1][127]=-np.inf
    bct[1][128]=-np.inf
    bct[1][129]=-np.inf
    bct[1][130]=-np.inf

    bct[2][0]=-1.1944937003628249
    bct[2][1]=-1.2587681978190854
    bct[2][2]=-1.3227920738605798
    bct[2][3]=-1.388915384611205
    bct[2][4]=-1.4551675897002387
    bct[2][5]=-1.527978460921148
    bct[2][6]=-1.6058561728753706
    bct[2][7]=-1.684785648364281
    bct[2][8]=-1.7661570957017574
    bct[2][9]=-1.8446888854024133
    bct[2][10]=-1.9235987762103244
    bct[2][11]=-1.9986953691171572
    bct[2][12]=-2.074994546781418
    bct[2][13]=-2.149695378420581
    bct[2][14]=-2.2242253161909917
    bct[2][15]=-2.2996493972899095
    bct[2][16]=-2.377020313124151
    bct[2][17]=-2.4559731465163503
    bct[2][18]=-2.5349259799085493
    bct[2][19]=-2.6153269939782176
    bct[2][20]=-2.6928675957633836
    bct[2][21]=-2.774163914841278
    bct[2][22]=-2.855272486297252
    bct[2][23]=-2.936910991681731
    bct[2][24]=-3.025773533293651
    bct[2][25]=-3.12016230694701
    bct[2][26]=-3.219906057710749
    bct[2][27]=-3.320597159192505
    bct[2][28]=-3.4213155407433646
    bct[2][29]=-3.528563447643646
    bct[2][30]=-3.641723531009429
    bct[2][31]=-3.7548915152190476
    bct[2][32]=-3.8686426971596606
    bct[2][33]=-3.982393879100275
    bct[2][34]=-4.0956730099110255
    bct[2][35]=-4.206389657972035
    bct[2][36]=-4.316569037225167
    bct[2][37]=-4.425903533314796
    bct[2][38]=-4.535176189498572
    bct[2][39]=-4.644448845682347
    bct[2][40]=-4.7537215018661225
    bct[2][41]=-4.858865134270627
    bct[2][42]=-4.962524236623965
    bct[2][43]=-5.0624884548409055
    bct[2][44]=-5.1618530911130565
    bct[2][45]=-5.2555467168267995
    bct[2][46]=-5.349240342540544
    bct[2][47]=-5.442933968254287
    bct[2][48]=-5.538658652281352
    bct[2][49]=-5.6396351490741505
    bct[2][50]=-5.747115589116515
    bct[2][51]=-5.8561808767259595
    bct[2][52]=-5.965973243328374
    bct[2][53]=-6.071640876865919
    bct[2][54]=-6.1773085104034635
    bct[2][55]=-6.282976143941006
    bct[2][56]=-6.388643777478551
    bct[2][57]=-6.4807562482465375
    bct[2][58]=-6.55052247635019
    bct[2][59]=-6.6074480898172006
    bct[2][60]=-6.655176147902994
    bct[2][61]=-6.694859381088254
    bct[2][62]=-6.732494921565965
    bct[2][63]=-6.770130462043676
    bct[2][64]=-6.8077660025213875
    bct[2][65]=-6.845401542999099
    bct[2][66]=-6.875673355011166
    bct[2][67]=-6.901696811016439
    bct[2][68]=-6.939822092017933
    bct[2][69]=-6.977872586197361
    bct[2][70]=-7.014579062386765
    bct[2][71]=-7.0446929462260375
    bct[2][72]=-7.073420001255991
    bct[2][73]=-7.1021470562859434
    bct[2][74]=-7.130874111315897
    bct[2][75]=-7.159601166345851
    bct[2][76]=-7.189679800748022
    bct[2][77]=-7.223679259163889
    bct[2][78]=-7.2460749788223815
    bct[2][79]=-7.253710131377538
    bct[2][80]=-7.259543344728238
    bct[2][81]=-7.253734397401479
    bct[2][82]=-7.247621181246449
    bct[2][83]=-7.241423303114622
    bct[2][84]=-7.235225424982794
    bct[2][85]=-7.229027546850967
    bct[2][86]=-7.22282966871914
    bct[2][87]=-7.216631790587312
    bct[2][88]=-7.205936871373936
    bct[2][89]=-7.1919894138482565
    bct[2][90]=-7.185842893891987
    bct[2][91]=-7.185946396701555
    bct[2][92]=-7.186151067967263
    bct[2][93]=-7.1922192758203485
    bct[2][94]=-7.1990433593525385
    bct[2][95]=-7.2067846812335095
    bct[2][96]=-7.2145260031144804
    bct[2][97]=-7.222267324995451
    bct[2][98]=-7.230008646876422
    bct[2][99]=-7.237749968757392
    bct[2][100]=-7.245491290638363
    bct[2][101]=-7.254739463313834
    bct[2][102]=-7.264506279593778
    bct[2][103]=-7.2661632263056255
    bct[2][104]=-7.260072731489626
    bct[2][105]=-7.253982236673626
    bct[2][106]=-7.245512143821606
    bct[2][107]=-7.231543095971819
    bct[2][108]=-7.216694781780314
    bct[2][109]=-7.20101143860175
    bct[2][110]=-7.185328095423185
    bct[2][111]=-7.169644752244622
    bct[2][112]=-7.153961409066058
    bct[2][113]=-7.138278065887494
    bct[2][114]=-7.1225947227089295
    bct[2][115]=-7.106911379530366
    bct[2][116]=-7.090464326688241
    bct[2][117]=-7.074489314668876
    bct[2][118]=-7.062692041670464
    bct[2][119]=-7.058161542607227
    bct[2][120]=-7.053631043543992
    bct[2][121]=-7.049100544480756
    bct[2][122]=-7.049737835924413
    bct[2][123]=-7.050431268211568
    bct[2][124]=-7.052330981493977
    bct[2][125]=-7.054273545650675
    bct[2][126]=-7.056216109807375
    bct[2][127]=-7.0581586739640745
    bct[2][128]=-7.060101238120772
    bct[2][129]=-7.062043802277472
    bct[2][130]=-7.063986366434171

    bct[3][0]=0.38609523271530216
    bct[3][1]=0.3490210969018331
    bct[3][2]=0.31057498662270305
    bct[3][3]=0.2694773567529319
    bct[3][4]=0.22701723497647697
    bct[3][5]=0.18380374510443853
    bct[3][6]=0.14060448846295434
    bct[3][7]=0.09763967609161975
    bct[3][8]=0.05307478707418645
    bct[3][9]=0.01030749728379133
    bct[3][10]=-0.031946598905093805
    bct[3][11]=-0.0758986518415461
    bct[3][12]=-0.12365134573734138
    bct[3][13]=-0.17097398616022552
    bct[3][14]=-0.2183959209955796
    bct[3][15]=-0.2659705958769188
    bct[3][16]=-0.31132013433265227
    bct[3][17]=-0.35662705400566125
    bct[3][18]=-0.40193397367867
    bct[3][19]=-0.4466297858088606
    bct[3][20]=-0.4917310399645575
    bct[3][21]=-0.5397781664015691
    bct[3][22]=-0.588430154572423
    bct[3][23]=-0.6378045763864683
    bct[3][24]=-0.6898148895119767
    bct[3][25]=-0.7451464016649201
    bct[3][26]=-0.8044086353725005
    bct[3][27]=-0.8640312461046007
    bct[3][28]=-0.9236603951785824
    bct[3][29]=-0.9859576338032234
    bct[3][30]=-1.0515319968238648
    bct[3][31]=-1.1192546872096087
    bct[3][32]=-1.1881641929285152
    bct[3][33]=-1.2570736986474218
    bct[3][34]=-1.3257245607167376
    bct[3][35]=-1.394789419526087
    bct[3][36]=-1.4648079585424982
    bct[3][37]=-1.5368456448735497
    bct[3][38]=-1.608647974737997
    bct[3][39]=-1.6804503046024433
    bct[3][40]=-1.7522526344668898
    bct[3][41]=-1.8223690526108802
    bct[3][42]=-1.8935701887443408
    bct[3][43]=-1.9632819693760268
    bct[3][44]=-2.0327192469485023
    bct[3][45]=-2.099560199263639
    bct[3][46]=-2.1664011515787758
    bct[3][47]=-2.2332421038939123
    bct[3][48]=-2.2996525135469272
    bct[3][49]=-2.3649496467202353
    bct[3][50]=-2.4279602060559258
    bct[3][51]=-2.4892070343011437
    bct[3][52]=-2.5457229737729286
    bct[3][53]=-2.6010454996394867
    bct[3][54]=-2.656368025506044
    bct[3][55]=-2.7116905513726017
    bct[3][56]=-2.76701307723916
    bct[3][57]=-2.8210760563508317
    bct[3][58]=-2.8735368108686323
    bct[3][59]=-2.9258983728498116
    bct[3][60]=-2.978658546999684
    bct[3][61]=-3.0316585787284582
    bct[3][62]=-3.084669668629525
    bct[3][63]=-3.1376807585305935
    bct[3][64]=-3.1906918484316615
    bct[3][65]=-3.2437029383327287
    bct[3][66]=-3.2981855175517882
    bct[3][67]=-3.353632456422372
    bct[3][68]=-3.4082206725724378
    bct[3][69]=-3.46285827879853
    bct[3][70]=-3.5183834899378326
    bct[3][71]=-3.574941858983337
    bct[3][72]=-3.631717564862788
    bct[3][73]=-3.688493270742242
    bct[3][74]=-3.7452689766216944
    bct[3][75]=-3.8020446825011462
    bct[3][76]=-3.856898958624713
    bct[3][77]=-3.9061793201988415
    bct[3][78]=-3.955543153117568
    bct[3][79]=-4.005013165984747
    bct[3][80]=-4.054350704219121
    bct[3][81]=-4.102832336240345
    bct[3][82]=-4.14682597229144
    bct[3][83]=-4.189570835627795
    bct[3][84]=-4.2323156989641495
    bct[3][85]=-4.275060562300505
    bct[3][86]=-4.317805425636861
    bct[3][87]=-4.360550288973216
    bct[3][88]=-4.4046755450637045
    bct[3][89]=-4.449799186589734
    bct[3][90]=-4.49238567713181
    bct[3][91]=-4.532939431061269
    bct[3][92]=-4.573398955606409
    bct[3][93]=-4.60839711933168
    bct[3][94]=-4.643638213917059
    bct[3][95]=-4.679174099705746
    bct[3][96]=-4.714709985494432
    bct[3][97]=-4.750245871283118
    bct[3][98]=-4.785781757071804
    bct[3][99]=-4.82131764286049
    bct[3][100]=-4.856853528649175
    bct[3][101]=-4.888757926634216
    bct[3][102]=-4.916987064176154
    bct[3][103]=-4.940036307389199
    bct[3][104]=-4.959371822966301
    bct[3][105]=-4.978707338543405
    bct[3][106]=-4.997052004628704
    bct[3][107]=-5.013106940817392
    bct[3][108]=-5.026369190179245
    bct[3][109]=-5.036979257441615
    bct[3][110]=-5.047589324703985
    bct[3][111]=-5.058199391966355
    bct[3][112]=-5.068809459228724
    bct[3][113]=-5.079419526491096
    bct[3][114]=-5.090029593753465
    bct[3][115]=-5.100639661015835
    bct[3][116]=-5.110226177346343
    bct[3][117]=-5.1181705116419
    bct[3][118]=-5.119712402367194
    bct[3][119]=-5.12097149374996
    bct[3][120]=-5.1222305851327254
    bct[3][121]=-5.123489676515492
    bct[3][122]=-5.1203056051913265
    bct[3][123]=-5.117073265095076
    bct[3][124]=-5.1112222253529085
    bct[3][125]=-5.105278161207828
    bct[3][126]=-5.099334097062746
    bct[3][127]=-5.0933900329176645
    bct[3][128]=-5.087445968772583
    bct[3][129]=-5.081501904627501
    bct[3][130]=-5.0755578404824195

    bct[4][0]=1.5261441422016604
    bct[4][1]=1.508165936913107
    bct[4][2]=1.4889063172270438
    bct[4][3]=1.4664426010458873
    bct[4][4]=1.4421031329434644
    bct[4][5]=1.4167195513030892
    bct[4][6]=1.3887393762249616
    bct[4][7]=1.3597066243370834
    bct[4][8]=1.3291317661608713
    bct[4][9]=1.2985443258175313
    bct[4][10]=1.2687211074246822
    bct[4][11]=1.2405716150782125
    bct[4][12]=1.2148501337795148
    bct[4][13]=1.191153283835013
    bct[4][14]=1.1675109155940742
    bct[4][15]=1.144323452216552
    bct[4][16]=1.1231996212471234
    bct[4][17]=1.1051446192420544
    bct[4][18]=1.0870896172369855
    bct[4][19]=1.0721101768225534
    bct[4][20]=1.059960300227151
    bct[4][21]=1.0473484709127405
    bct[4][22]=1.034793327830716
    bct[4][23]=1.0213630240591374
    bct[4][24]=1.0054309905898768
    bct[4][25]=0.9880320572325879
    bct[4][26]=0.9674434425251089
    bct[4][27]=0.9463683893287356
    bct[4][28]=0.925277936485467
    bct[4][29]=0.9010618197487862
    bct[4][30]=0.8705627554949844
    bct[4][31]=0.8352612054423265
    bct[4][32]=0.796983023080082
    bct[4][33]=0.7587048407178374
    bct[4][34]=0.7197928707240459
    bct[4][35]=0.6769101840275943
    bct[4][36]=0.6329421244666099
    bct[4][37]=0.5832577961700479
    bct[4][38]=0.5299544715883221
    bct[4][39]=0.4766511470065966
    bct[4][40]=0.4233478224248713
    bct[4][41]=0.3698748558594276
    bct[4][42]=0.30985919901828507
    bct[4][43]=0.24892067830215356
    bct[4][44]=0.18793272506171355
    bct[4][45]=0.12647722544214343
    bct[4][46]=0.06502172582257275
    bct[4][47]=0.0035662262030025983
    bct[4][48]=-0.05746759872553695
    bct[4][49]=-0.11741107754691467
    bct[4][50]=-0.1775326134896975
    bct[4][51]=-0.23689336124857047
    bct[4][52]=-0.29393376844369973
    bct[4][53]=-0.350167396363407
    bct[4][54]=-0.4064010242831138
    bct[4][55]=-0.4626346522028206
    bct[4][56]=-0.518868280122528
    bct[4][57]=-0.572926865872799
    bct[4][58]=-0.6258640448269991
    bct[4][59]=-0.6780030006056735
    bct[4][60]=-0.7296246920866905
    bct[4][61]=-0.7806659139608182
    bct[4][62]=-0.8315005516532242
    bct[4][63]=-0.8823351893456322
    bct[4][64]=-0.9331698270380392
    bct[4][65]=-0.984004464730446
    bct[4][66]=-1.0355883668538979
    bct[4][67]=-1.088905105619253
    bct[4][68]=-1.1417253670462373
    bct[4][69]=-1.194524749980565
    bct[4][70]=-1.246948918820226
    bct[4][71]=-1.2995989485898356
    bct[4][72]=-1.352296490844353
    bct[4][73]=-1.404994033098872
    bct[4][74]=-1.4576915753533903
    bct[4][75]=-1.5103891176079085
    bct[4][76]=-1.56293853226573
    bct[4][77]=-1.6150582405739542
    bct[4][78]=-1.6672873071131158
    bct[4][79]=-1.7196554830841402
    bct[4][80]=-1.7722794131745574
    bct[4][81]=-1.826555746711513
    bct[4][82]=-1.8804341826620163
    bct[4][83]=-1.9342019046875223
    bct[4][84]=-1.9879696267130282
    bct[4][85]=-2.041737348738534
    bct[4][86]=-2.0955050707640406
    bct[4][87]=-2.149272792789547
    bct[4][88]=-2.2007584790729786
    bct[4][89]=-2.2505936559333137
    bct[4][90]=-2.300072849606668
    bct[4][91]=-2.349266833585278
    bct[4][92]=-2.398431929173931
    bct[4][93]=-2.4459227071152045
    bct[4][94]=-2.4931434710975906
    bct[4][95]=-2.54003657913781
    bct[4][96]=-2.5869296871780296
    bct[4][97]=-2.6338227952182494
    bct[4][98]=-2.6807159032584686
    bct[4][99]=-2.727609011298688
    bct[4][100]=-2.774502119338907
    bct[4][101]=-2.8214274463057127
    bct[4][102]=-2.8681488577269842
    bct[4][103]=-2.912633723308642
    bct[4][104]=-2.955911816696697
    bct[4][105]=-2.999189910084752
    bct[4][106]=-3.0407015842617438
    bct[4][107]=-3.0781312834153063
    bct[4][108]=-3.112636335708578
    bct[4][109]=-3.144363884998061
    bct[4][110]=-3.1760914342875446
    bct[4][111]=-3.207818983577029
    bct[4][112]=-3.239546532866512
    bct[4][113]=-3.271274082155996
    bct[4][114]=-3.3030016314454786
    bct[4][115]=-3.3347291807349624
    bct[4][116]=-3.3631507037561477
    bct[4][117]=-3.3902148845499656
    bct[4][118]=-3.414339409400906
    bct[4][119]=-3.4351306510514075
    bct[4][120]=-3.4559218927019075
    bct[4][121]=-3.4767131343524094
    bct[4][122]=-3.4932599920594507
    bct[4][123]=-3.5097607404487103
    bct[4][124]=-3.5247362980922747
    bct[4][125]=-3.539657676189641
    bct[4][126]=-3.554579054287007
    bct[4][127]=-3.569500432384372
    bct[4][128]=-3.5844218104817376
    bct[4][129]=-3.5993431885791027
    bct[4][130]=-3.614264566676469

    bct[5][0]=0.8625687785530236
    bct[5][1]=0.8998257468116381
    bct[5][2]=0.9369449781061147
    bct[5][3]=0.9720168440102466
    bct[5][4]=1.0062838165920958
    bct[5][5]=1.04111429270014
    bct[5][6]=1.0761040215447266
    bct[5][7]=1.1114053175775311
    bct[5][8]=1.147331369834863
    bct[5][9]=1.181506883417438
    bct[5][10]=1.2145469561991766
    bct[5][11]=1.2449145635045842
    bct[5][12]=1.2713249576512686
    bct[5][13]=1.2944347207812958
    bct[5][14]=1.3174277337799887
    bct[5][15]=1.3369575122714545
    bct[5][16]=1.3519712127971482
    bct[5][17]=1.3634596670877501
    bct[5][18]=1.3749481213783519
    bct[5][19]=1.3829238611452839
    bct[5][20]=1.3872122338700579
    bct[5][21]=1.3894107689916444
    bct[5][22]=1.3912852882141085
    bct[5][23]=1.3929560093883209
    bct[5][24]=1.3910064205831247
    bct[5][25]=1.3872194001935432
    bct[5][26]=1.3818855699917274
    bct[5][27]=1.376187423344631
    bct[5][28]=1.3704802289595426
    bct[5][29]=1.362360264737382
    bct[5][30]=1.3502818239901122
    bct[5][31]=1.3372705039866082
    bct[5][32]=1.3235490362075926
    bct[5][33]=1.309827568428577
    bct[5][34]=1.2956945389892063
    bct[5][35]=1.279257685515189
    bct[5][36]=1.2623045595892812
    bct[5][37]=1.241468650931047
    bct[5][38]=1.2181934788277056
    bct[5][39]=1.1949183067243638
    bct[5][40]=1.1716431346210225
    bct[5][41]=1.1479277610167056
    bct[5][42]=1.1213710707459794
    bct[5][43]=1.094630768637213
    bct[5][44]=1.0677796767936127
    bct[5][45]=1.0398807052373864
    bct[5][46]=1.0119817336811594
    bct[5][47]=0.9840827621249328
    bct[5][48]=0.9559802732977968
    bct[5][49]=0.9273515393134443
    bct[5][50]=0.8975904936568851
    bct[5][51]=0.8675749432965557
    bct[5][52]=0.8367517360346443
    bct[5][53]=0.8054294308729969
    bct[5][54]=0.7741071257113498
    bct[5][55]=0.7427848205497027
    bct[5][56]=0.7114625153880554
    bct[5][57]=0.6795932360080764
    bct[5][58]=0.6463376014618795
    bct[5][59]=0.6125324378366542
    bct[5][60]=0.5787948144146225
    bct[5][61]=0.5450418041756591
    bct[5][62]=0.511250660639022
    bct[5][63]=0.47745951710238377
    bct[5][64]=0.44366837356574607
    bct[5][65]=0.4098772300291085
    bct[5][66]=0.3764201593918445
    bct[5][67]=0.34175938321083327
    bct[5][68]=0.3080472144215468
    bct[5][69]=0.2744146600659149
    bct[5][70]=0.24221288225810006
    bct[5][71]=0.21051690778598356
    bct[5][72]=0.1789273349538516
    bct[5][73]=0.14733776212171854
    bct[5][74]=0.11574818928958602
    bct[5][75]=0.08415861645745348
    bct[5][76]=0.05293334519951929
    bct[5][77]=0.02276488376697168
    bct[5][78]=-0.007206607741498949
    bct[5][79]=-0.0369275431281463
    bct[5][80]=-0.06664523360561768
    bct[5][81]=-0.0963419590283765
    bct[5][82]=-0.12534634905935235
    bct[5][83]=-0.15415809864402139
    bct[5][84]=-0.18296984822869033
    bct[5][85]=-0.2117815978133594
    bct[5][86]=-0.24059334739802837
    bct[5][87]=-0.2694050969826979
    bct[5][88]=-0.29726320045408955
    bct[5][89]=-0.32443156806059353
    bct[5][90]=-0.351067877979208
    bct[5][91]=-0.37727790930855565
    bct[5][92]=-0.4034588891672603
    bct[5][93]=-0.4279560995237458
    bct[5][94]=-0.4516421673210853
    bct[5][95]=-0.4743439316381764
    bct[5][96]=-0.497045695955268
    bct[5][97]=-0.5197474602723592
    bct[5][98]=-0.5424492245894502
    bct[5][99]=-0.5651509889065415
    bct[5][100]=-0.5878527532236326
    bct[5][101]=-0.6102005874289044
    bct[5][102]=-0.6318113098101996
    bct[5][103]=-0.6492418654946186
    bct[5][104]=-0.6641873275616836
    bct[5][105]=-0.6791327896287488
    bct[5][106]=-0.6931774065911833
    bct[5][107]=-0.7051402825976489
    bct[5][108]=-0.7161985153328854
    bct[5][109]=-0.7263976189071579
    bct[5][110]=-0.7365967224814306
    bct[5][111]=-0.7467958260557035
    bct[5][112]=-0.756994929629976
    bct[5][113]=-0.7671940332042487
    bct[5][114]=-0.7773931367785214
    bct[5][115]=-0.787592240352794
    bct[5][116]=-0.7962676911651713
    bct[5][117]=-0.8042843295004606
    bct[5][118]=-0.8110255560009487
    bct[5][119]=-0.8177213776727413
    bct[5][120]=-0.8244171993445338
    bct[5][121]=-0.8311130210163266
    bct[5][122]=-0.8370579955126625
    bct[5][123]=-0.8429948131006036
    bct[5][124]=-0.8475889470383994
    bct[5][125]=-0.8521353846529259
    bct[5][126]=-0.8566818222674524
    bct[5][127]=-0.8612282598819789
    bct[5][128]=-0.8657746974965055
    bct[5][129]=-0.8703211351110316
    bct[5][130]=-0.8748675727255582

    bct[6][0]=-0.8031771907732801
    bct[6][1]=-0.6616432769806067
    bct[6][2]=-0.5229297334326294
    bct[6][3]=-0.3938712250438224
    bct[6][4]=-0.2692090342217983
    bct[6][5]=-0.14880741925208732
    bct[6][6]=-0.042392617999716456
    bct[6][7]=0.05999937114847218
    bct[6][8]=0.15125182769451798
    bct[6][9]=0.22529162269828318
    bct[6][10]=0.2949905276639512
    bct[6][11]=0.35725034824180146
    bct[6][12]=0.40836641605789503
    bct[6][13]=0.4530337540988182
    bct[6][14]=0.4974400025347985
    bct[6][15]=0.533560175629891
    bct[6][16]=0.5605307596774118
    bct[6][17]=0.5842683820449969
    bct[6][18]=0.6080060044125819
    bct[6][19]=0.6272935405256543
    bct[6][20]=0.6427301587864034
    bct[6][21]=0.6570429256614783
    bct[6][22]=0.6710503996930787
    bct[6][23]=0.6848572574898931
    bct[6][24]=0.6951793407257537
    bct[6][25]=0.7052614866591729
    bct[6][26]=0.7154786811157541
    bct[6][27]=0.7254339840668597
    bct[6][28]=0.735393060725723
    bct[6][29]=0.7455778733036434
    bct[6][30]=0.7536261288286208
    bct[6][31]=0.765132268872708
    bct[6][32]=0.7777225768669411
    bct[6][33]=0.7903128848611738
    bct[6][34]=0.8027403126390049
    bct[6][35]=0.816391318986692
    bct[6][36]=0.8313039037583483
    bct[6][37]=0.8423334777949201
    bct[6][38]=0.8524615387490426
    bct[6][39]=0.8625895997031652
    bct[6][40]=0.872717660657288
    bct[6][41]=0.8825206088603137
    bct[6][42]=0.8929860283470763
    bct[6][43]=0.9043703190746943
    bct[6][44]=0.915648774376403
    bct[6][45]=0.9259262091787491
    bct[6][46]=0.9362036439810952
    bct[6][47]=0.9464810787834413
    bct[6][48]=0.9564094137894618
    bct[6][49]=0.965435063355662
    bct[6][50]=0.9729934642218909
    bct[6][51]=0.980041656424586
    bct[6][52]=0.9854419994136597
    bct[6][53]=0.9899132124422967
    bct[6][54]=0.9943844254709332
    bct[6][55]=0.99885563849957
    bct[6][56]=1.0033268515282068
    bct[6][57]=1.0075327176565485
    bct[6][58]=1.0089294091353798
    bct[6][59]=1.0098891048766732
    bct[6][60]=1.009893680551557
    bct[6][61]=1.0091526470945904
    bct[6][62]=1.0082630971364777
    bct[6][63]=1.0073735471783647
    bct[6][64]=1.0064839972202515
    bct[6][65]=1.0055944472621385
    bct[6][66]=1.004082022228197
    bct[6][67]=0.9987475363655638
    bct[6][68]=0.9921505821022021
    bct[6][69]=0.9855157703904983
    bct[6][70]=0.9782006103203108
    bct[6][71]=0.9695629150163863
    bct[6][72]=0.9606470089774322
    bct[6][73]=0.9517311029384775
    bct[6][74]=0.9428151968995233
    bct[6][75]=0.933899290860569
    bct[6][76]=0.9249035164264161
    bct[6][77]=0.915676050144737
    bct[6][78]=0.906107681815421
    bct[6][79]=0.8961056681192958
    bct[6][80]=0.8859428666142528
    bct[6][81]=0.8747412301131305
    bct[6][82]=0.8636233420756178
    bct[6][83]=0.8525287568211186
    bct[6][84]=0.8414341715666194
    bct[6][85]=0.8303395863121203
    bct[6][86]=0.8192450010576211
    bct[6][87]=0.8081504158031217
    bct[6][88]=0.7971780826909902
    bct[6][89]=0.7862941698911485
    bct[6][90]=0.7753530042047259
    bct[6][91]=0.764365968154471
    bct[6][92]=0.7533826517084928
    bct[6][93]=0.7426149166462809
    bct[6][94]=0.7318240842178161
    bct[6][95]=0.72100522364839
    bct[6][96]=0.710186363078964
    bct[6][97]=0.699367502509538
    bct[6][98]=0.6885486419401121
    bct[6][99]=0.6777297813706862
    bct[6][100]=0.6669109208012602
    bct[6][101]=0.656468863283211
    bct[6][102]=0.646501339832352
    bct[6][103]=0.6379291227921996
    bct[6][104]=0.629926775752191
    bct[6][105]=0.6219244287121823
    bct[6][106]=0.6141503833560539
    bct[6][107]=0.6069039147866637
    bct[6][108]=0.5997449523582546
    bct[6][109]=0.5926690934910738
    bct[6][110]=0.5855932346238928
    bct[6][111]=0.5785173757567117
    bct[6][112]=0.5714415168895307
    bct[6][113]=0.5643656580223498
    bct[6][114]=0.557289799155169
    bct[6][115]=0.5502139402879881
    bct[6][116]=0.5431312638203162
    bct[6][117]=0.5361746733845015
    bct[6][118]=0.5297200017551699
    bct[6][119]=0.5227211696003639
    bct[6][120]=0.5157223374455578
    bct[6][121]=0.5087235052907516
    bct[6][122]=0.5024655813732288
    bct[6][123]=0.49621570639138535
    bct[6][124]=0.49062833105232573
    bct[6][125]=0.485064489772927
    bct[6][126]=0.479500648493528
    bct[6][127]=0.4739368072141293
    bct[6][128]=0.4683729659347304
    bct[6][129]=0.46280912465533164
    bct[6][130]=0.4572452833759328

    bct[7][0]=-2.7570319209201695
    bct[7][1]=-2.5745104153822767
    bct[7][2]=-2.394426115465908
    bct[7][3]=-2.22602049300832
    bct[7][4]=-2.0662313168773636
    bct[7][5]=-1.9035842532327483
    bct[7][6]=-1.7360276977751554
    bct[7][7]=-1.569361389265861
    bct[7][8]=-1.4140983373221876
    bct[7][9]=-1.2748880356865127
    bct[7][10]=-1.1402766521367182
    bct[7][11]=-1.0091428848194288
    bct[7][12]=-0.8837193250002313
    bct[7][13]=-0.7688631391427126
    bct[7][14]=-0.6545802905462379
    bct[7][15]=-0.5437966763862305
    bct[7][16]=-0.43431566246498376
    bct[7][17]=-0.3240638288260254
    bct[7][18]=-0.21381199518706753
    bct[7][19]=-0.10838903384601349
    bct[7][20]=-0.009006232633358247
    bct[7][21]=0.09145389352765738
    bct[7][22]=0.19195510337468666
    bct[7][23]=0.29198047012213063
    bct[7][24]=0.38946729845563227
    bct[7][25]=0.4856369695616392
    bct[7][26]=0.5796745441210371
    bct[7][27]=0.6733970449946768
    bct[7][28]=0.7670986961267448
    bct[7][29]=0.8561453109937323
    bct[7][30]=0.9387634052049503
    bct[7][31]=1.0134070138571047
    bct[7][32]=1.0852048177231237
    bct[7][33]=1.1570026215891425
    bct[7][34]=1.2278826824159788
    bct[7][35]=1.2933514157377075
    bct[7][36]=1.3574807974983523
    bct[7][37]=1.4112311698545068
    bct[7][38]=1.4582534847663746
    bct[7][39]=1.5052757996782415
    bct[7][40]=1.5522981145901085
    bct[7][41]=1.5976384809628437
    bct[7][42]=1.6296264146628645
    bct[7][43]=1.658549999022779
    bct[7][44]=1.6871817708217762
    bct[7][45]=1.713053499346591
    bct[7][46]=1.7389252278714047
    bct[7][47]=1.7647969563962194
    bct[7][48]=1.7904644286525333
    bct[7][49]=1.8156037448872517
    bct[7][50]=1.839351647220738
    bct[7][51]=1.862802857217034
    bct[7][52]=1.8855277026043737
    bct[7][53]=1.9080177817256212
    bct[7][54]=1.9305078608468678
    bct[7][55]=1.9529979399681148
    bct[7][56]=1.9754880190893616
    bct[7][57]=1.996481210926126
    bct[7][58]=2.0133013002663547
    bct[7][59]=2.029734702892299
    bct[7][60]=2.0456216388527397
    bct[7][61]=2.0610484066297055
    bct[7][62]=2.0763662292426175
    bct[7][63]=2.0916840518555304
    bct[7][64]=2.1070018744684433
    bct[7][65]=2.1223196970813554
    bct[7][66]=2.1368822367224993
    bct[7][67]=2.149434800001257
    bct[7][68]=2.161569925548716
    bct[7][69]=2.1736460057969706
    bct[7][70]=2.1846609640164267
    bct[7][71]=2.194935430035942
    bct[7][72]=2.205054124871465
    bct[7][73]=2.2151728197069884
    bct[7][74]=2.2252915145425116
    bct[7][75]=2.2354102093780344
    bct[7][76]=2.245411880915977
    bct[7][77]=2.2550740771964906
    bct[7][78]=2.264240050111333
    bct[7][79]=2.272774800758905
    bct[7][80]=2.281201102984583
    bct[7][81]=2.288926730098809
    bct[7][82]=2.2956495192988124
    bct[7][83]=2.302093271567204
    bct[7][84]=2.3085370238355964
    bct[7][85]=2.314980776103989
    bct[7][86]=2.3214245283723813
    bct[7][87]=2.327868280640774
    bct[7][88]=2.3339010884753555
    bct[7][89]=2.3396366758729976
    bct[7][90]=2.344955674667918
    bct[7][91]=2.349940907395896
    bct[7][92]=2.3549105074868932
    bct[7][93]=2.358974068853997
    bct[7][94]=2.3628144280078778
    bct[7][95]=2.3663839362332766
    bct[7][96]=2.3699534444586763
    bct[7][97]=2.3735229526840755
    bct[7][98]=2.377092460909475
    bct[7][99]=2.3806619691348745
    bct[7][100]=2.3842314773602737
    bct[7][101]=2.387619419818451
    bct[7][102]=2.390977632064747
    bct[7][103]=2.3954785322805416
    bct[7][104]=2.399980155855516
    bct[7][105]=2.404481779430491
    bct[7][106]=2.408860113083282
    bct[7][107]=2.41295353906532
    bct[7][108]=2.417086255022269
    bct[7][109]=2.421256284210027
    bct[7][110]=2.4254263133977845
    bct[7][111]=2.4295963425855422
    bct[7][112]=2.4337663717733005
    bct[7][113]=2.4379364009610582
    bct[7][114]=2.442106430148816
    bct[7][115]=2.446276459336574
    bct[7][116]=2.450358172847835
    bct[7][117]=2.454302418109301
    bct[7][118]=2.457592129102455
    bct[7][119]=2.460076759473851
    bct[7][120]=2.4625613898452468
    bct[7][121]=2.465046020216643
    bct[7][122]=2.4670739167754094
    bct[7][123]=2.469096851557675
    bct[7][124]=2.4704617934678703
    bct[7][125]=2.4718033614129973
    bct[7][126]=2.4731449293581242
    bct[7][127]=2.4744864973032517
    bct[7][128]=2.4758280652483786
    bct[7][129]=2.4771696331935056
    bct[7][130]=2.4785112011386325

    return td, bct

register_model('DiUj2017', KNTable, get_DiUj2017_model,
                 usage="table")
